@@ -20,7 +20,7 @@
 
 #include <iostream>
 #include <fstream>
-#include "pugixml/pugixml.hpp"  //pugi XML parser
+#include <pugixml.hpp>  //pugi XML parser
 #include <sstream>
 #include <algorithm>
 #include "smarc/smarc.h"      //resampling library
@@ -85,7 +85,7 @@ int Xdf::load_xdf(std::string filename)
                 break;
 
             uint16_t tag;   //read tag of the chunk, 6 possibilities
-            file.read(reinterpret_cast<char *>(&tag), 2);
+            readBin(file, &tag);
 
             switch (tag)
             {
@@ -109,9 +109,9 @@ int Xdf::load_xdf(std::string filename)
             case 2: //read [StreamHeader] chunk
             {
                 //read [StreamID]
-                int streamID;
+                uint32_t streamID;
                 int index;
-                file.read((char*)&streamID, 4);
+                Xdf::readBin(file, &streamID);
                 std::vector<int>::iterator it {std::find(idmap.begin(),idmap.end(),streamID)};
                 if (it == idmap.end())
                 {
@@ -160,9 +160,9 @@ int Xdf::load_xdf(std::string filename)
             case 3: //read [Samples] chunk
             {
                 //read [StreamID]
-                int streamID;
+                uint32_t streamID;
                 int index;
-                file.read((char*)&streamID, 4);
+                Xdf::readBin(file, &streamID);
                 std::vector<int>::iterator it {std::find(idmap.begin(),idmap.end(),streamID)};
                 if (it == idmap.end())
                 {
@@ -188,14 +188,13 @@ int Xdf::load_xdf(std::string filename)
                     for (size_t i = 0; i < numSamp; i++)
                     {
                         //read or deduce time stamp
-                        uint8_t tsBytes;
-                        file.read(reinterpret_cast<char *>(&tsBytes), 1);
+                        auto tsBytes = readBin<uint8_t>(file);
 
                         double ts;  //temporary time stamp
 
                         if (tsBytes == 8)
                         {
-                            file.read((char*)&ts, 8);
+                            Xdf::readBin(file, &ts);
                             streams[index].time_stamps.emplace_back(ts);
                         }
                         else
@@ -210,7 +209,7 @@ int Xdf::load_xdf(std::string filename)
                         for (int v = 0; v < streams[index].info.channel_count; ++v)
                         {
                             float data;
-                            file.read((char*)&data, 4);
+                            Xdf::readBin(file, &data);
                             streams[index].time_series[v].emplace_back(data);
                         }
                     }
@@ -225,14 +224,13 @@ int Xdf::load_xdf(std::string filename)
                     for (size_t i = 0; i < numSamp; i++)
                     {
                         //read or deduce time stamp
-                        uint8_t tsBytes;
-                        file.read(reinterpret_cast<char *>(&tsBytes), 1);
+                        auto tsBytes = readBin<uint8_t>(file);
 
                         double ts;  //temporary time stamp
 
                         if (tsBytes == 8)
                         {
-                            file.read((char*)&ts, 8);
+                            Xdf::readBin(file, &ts);
                             streams[index].time_stamps.emplace_back(ts);
                         }
                         else
@@ -247,7 +245,7 @@ int Xdf::load_xdf(std::string filename)
                         for (int v = 0; v < streams[index].info.channel_count; ++v)
                         {
                             double data;
-                            file.read((char*)&data, 8);
+                            Xdf::readBin(file, &data);
                             streams[index].time_series[v].emplace_back(data);
                         }
                     }
@@ -262,14 +260,13 @@ int Xdf::load_xdf(std::string filename)
                     for (size_t i = 0; i < numSamp; i++)
                     {
                         //read or deduce time stamp
-                        uint8_t tsBytes;
-                        file.read(reinterpret_cast<char *>(&tsBytes), 1);
+                        auto tsBytes = readBin<uint8_t>(file);
 
                         double ts;  //temporary time stamp
 
                         if (tsBytes == 8)
                         {
-                            file.read((char*)&ts, 8);
+                            Xdf::readBin(file, &ts);
                             streams[index].time_stamps.emplace_back(ts);
                         }
                         else
@@ -284,7 +281,7 @@ int Xdf::load_xdf(std::string filename)
                         for (int v = 0; v < streams[index].info.channel_count; ++v)
                         {
                             int8_t data;
-                            file.read((char*)&data, 1);
+                            Xdf::readBin(file, &data);
                             streams[index].time_series[v].emplace_back(data);
                         }
                     }
@@ -299,14 +296,13 @@ int Xdf::load_xdf(std::string filename)
                     for (size_t i = 0; i < numSamp; i++)
                     {
                         //read or deduce time stamp
-                        uint8_t tsBytes;
-                        file.read(reinterpret_cast<char *>(&tsBytes), 1);
+                        auto tsBytes = readBin<uint8_t>(file);
 
                         double ts;  //temporary time stamp
 
                         if (tsBytes == 8)
                         {
-                            file.read((char*)&ts, 8);
+                            Xdf::readBin(file, &ts);
                             streams[index].time_stamps.emplace_back(ts);
                         }
                         else
@@ -321,7 +317,7 @@ int Xdf::load_xdf(std::string filename)
                         for (int v = 0; v < streams[index].info.channel_count; ++v)
                         {
                             int16_t data;
-                            file.read((char*)&data, 2);
+                            Xdf::readBin(file, &data);
                             streams[index].time_series[v].emplace_back(data);
                         }
                     }
@@ -336,14 +332,13 @@ int Xdf::load_xdf(std::string filename)
                     for (size_t i = 0; i < numSamp; i++)
                     {
                         //read or deduce time stamp
-                        uint8_t tsBytes;
-                        file.read(reinterpret_cast<char *>(&tsBytes), 1);
+                        auto tsBytes = readBin<uint8_t>(file);
 
                         double ts;  //temporary time stamp
 
                         if (tsBytes == 8)
                         {
-                            file.read((char*)&ts, 8);
+                            Xdf::readBin(file, &ts);
                             streams[index].time_stamps.emplace_back(ts);
                         }
                         else
@@ -358,7 +353,7 @@ int Xdf::load_xdf(std::string filename)
                         for (int v = 0; v < streams[index].info.channel_count; ++v)
                         {
                             int32_t data;
-                            file.read((char*)&data, 4);
+                            Xdf::readBin(file, &data);
                             streams[index].time_series[v].emplace_back(data);
                         }
                     }
@@ -373,14 +368,13 @@ int Xdf::load_xdf(std::string filename)
                     for (size_t i = 0; i < numSamp; i++)
                     {
                         //read or deduce time stamp
-                        uint8_t tsBytes;
-                        file.read(reinterpret_cast<char *>(&tsBytes), 1);
+                        auto tsBytes = readBin<uint8_t>(file);
 
                         double ts;  //temporary time stamp
 
                         if (tsBytes == 8)
                         {
-                            file.read((char*)&ts, 8);
+                            Xdf::readBin(file, &ts);
                             streams[index].time_stamps.emplace_back(ts);
                         }
                         else
@@ -395,7 +389,7 @@ int Xdf::load_xdf(std::string filename)
                         for (int v = 0; v < streams[index].info.channel_count; ++v)
                         {
                             int64_t data;
-                            file.read((char*)&data, 8);
+                            Xdf::readBin(file, &data);
                             streams[index].time_series[v].emplace_back(data);
                         }
                     }
@@ -406,50 +400,23 @@ int Xdf::load_xdf(std::string filename)
                     for (size_t i = 0; i < numSamp; i++)
                     {
                         //read or deduce time stamp
-                        uint8_t tsBytes;
-                        file.read(reinterpret_cast<char *>(&tsBytes), 1);
+                        auto tsBytes = readBin<uint8_t>(file);
 
                         double ts;  //temporary time stamp
 
                         if (tsBytes == 8)
-                            file.read((char*)&ts, 8);
+                            Xdf::readBin(file, &ts);
                         else
                             ts = streams[index].last_timestamp + streams[index].sampling_interval;
 
                         //read the event
-                        int8_t bytes;
-                        file.read((char*)&bytes, 1);
-                        if (bytes == 1)
-                        {
-                            uint8_t length;
-                            file.read((char*)&length, 1);
-                            char* buffer = new char[length + 1];
-                            file.read(buffer, length);
-                            buffer[length] = '\0';
-                            eventMap.emplace_back(std::make_pair(buffer, ts), index);
-                            delete[] buffer;
-                        }
-                        else if (bytes == 4)
-                        {
-                            uint32_t length;
-                            file.read((char*)&length, 4);
-                            char* buffer = new char[length + 1];
-                            file.read(buffer, length);
-                            buffer[length] = '\0';
-                            eventMap.emplace_back(std::make_pair(buffer, ts), index);
-                            delete[] buffer;
-                        }
-                        else if (bytes == 8)
-                        {
-                            uint64_t length;
-                            file.read((char*)&length, 8);
-                            char* buffer = new char[length + 1];
-                            file.read(buffer, length);
-                            buffer[length] = '\0';
-                            eventMap.emplace_back(std::make_pair(buffer, ts), index);
-                            delete[] buffer;
-                        }
+                        auto length = Xdf::readLength(file);
 
+                        char* buffer = new char[length + 1];
+                        file.read(buffer, length);
+                        buffer[length] = '\0';
+                        eventMap.emplace_back(std::make_pair(buffer, ts), index);
+                        delete[] buffer;
                         streams[index].last_timestamp = ts;
                     }
                 }
@@ -457,9 +424,9 @@ int Xdf::load_xdf(std::string filename)
                 break;
             case 4: //read [ClockOffset] chunk
             {
-                int streamID;
+                uint32_t streamID;
                 int index;
-                file.read((char*)&streamID, 4);
+                Xdf::readBin(file, &streamID);
                 std::vector<int>::iterator it {std::find(idmap.begin(),idmap.end(),streamID)};
                 if (it == idmap.end())
                 {
@@ -474,8 +441,8 @@ int Xdf::load_xdf(std::string filename)
                 double collectionTime;
                 double offsetValue;
 
-                file.read((char*)&collectionTime, 8);
-                file.read((char*)&offsetValue, 8);
+                Xdf::readBin(file, &collectionTime);
+                Xdf::readBin(file, &offsetValue);
 
                 streams[index].clock_times.emplace_back(collectionTime);
                 streams[index].clock_values.emplace_back(offsetValue);
@@ -486,9 +453,9 @@ int Xdf::load_xdf(std::string filename)
                 pugi::xml_document doc;
 
                 //read [StreamID]
-                int streamID;
+                uint32_t streamID;
                 int index;
-                file.read((char*)&streamID, 4);
+                Xdf::readBin(file, &streamID);
                 std::vector<int>::iterator it {std::find(idmap.begin(),idmap.end(),streamID)};
                 if (it == idmap.end())
                 {
@@ -746,34 +713,27 @@ void Xdf::resample(int userSrate)
 uint64_t Xdf::readLength(std::ifstream &file)
 {
     uint8_t bytes;
-    file.read((char*)&bytes, 1);
+    Xdf::readBin(file, &bytes);
     uint64_t length = 0;
 
     switch (bytes)
     {
     case 1:
-    {
-        uint8_t buffer = 0;
-        file.read(reinterpret_cast<char *>(&buffer), 1);
-        length = buffer;
-        return length;
-    }
+        length = readBin<uint8_t>(file);
+        break;
     case 4:
-    {
-        uint32_t buffer = 0;
-        file.read(reinterpret_cast<char *>(&buffer), 4);
-        length = buffer;
-        return length;
-    }
+        length = readBin<uint32_t>(file);
+        break;
     case 8:
-    {
-        file.read(reinterpret_cast<char *>(&length), 8);
-        return length;
-    }
+        length = readBin<uint64_t>(file);
+        break;
     default:
-        std::cout << "Invalid variable-length integer encountered.\n";
+        std::cout << "Invalid variable-length integer length ("
+                  << static_cast<int>(bytes) << ") encountered.\n";
         return 0;
     }
+
+    return length;
 }
 
 void Xdf::findMinMax()
@@ -781,7 +741,7 @@ void Xdf::findMinMax()
     //find the smallest timestamp of all streams
     for (auto const &stream : streams)
     {
-        if (stream.info.first_timestamp != -1)
+        if (!std::isnan(stream.info.first_timestamp))
         {
             minTS = stream.info.first_timestamp;
             break;
@@ -789,14 +749,14 @@ void Xdf::findMinMax()
     }
     for (auto const &stream : streams)
     {
-        if (stream.info.first_timestamp != -1 && stream.info.first_timestamp < minTS)
+        if (!std::isnan(stream.info.first_timestamp) && stream.info.first_timestamp < minTS)
             minTS = stream.info.first_timestamp;
     }
 
     //find the max timestamp of all streams
     for (auto const &stream : streams)
     {
-        if (stream.info.last_timestamp != 0 && stream.info.last_timestamp > maxTS)
+        if (!std::isnan(stream.info.last_timestamp) && stream.info.last_timestamp > maxTS)
             maxTS = stream.info.last_timestamp;
     }
 }
@@ -915,7 +875,7 @@ void Xdf::detrend()
         {
             long double init = 0.0;
             long double mean = std::accumulate(row.begin(), row.end(), init) / row.size();
-            std::transform(row.begin(), row.end(), row.begin(), bind2nd(std::minus<double>(), mean));
+	    for(auto &val: row) val -= mean;
             offsets.emplace_back(mean);
         }
     }
@@ -1126,3 +1086,10 @@ void Xdf::loadDictionary()
             eventType.emplace_back(std::distance(dictionary.begin(), it));
     }
 }
+
+template<typename T> T Xdf::readBin(std::istream& is, T* obj) {
+	T dummy;
+	if(!obj) obj = &dummy;
+	is.read(reinterpret_cast<char*>(obj), sizeof(T));
+	return *obj;
+}
